from glLibLocals import *
from glLibShader import glLibShader, glLibUseShader
from glLibTexturing import glLibActiveTexture, glLibSelectTexture
from glLibMisc import *
from glLibView import *
from glLibObjects import glLibTexFullScreenQuad
def glLibBlankShader():
    Shader = glLibShader()
    return Shader
def glLibColorShader(color,intensity):
    Shader = glLibShader()
    if   color == GLLIB_RED:    color = [1,0,0]
    elif color == GLLIB_ORANGE: color = [1,0.5,0]
    elif color == GLLIB_YELLOW: color = [1,1,0]
    elif color == GLLIB_GREEN:  color = [0,1,0]
    elif color == GLLIB_BLUE:   color = [0,0,1]
    elif color == GLLIB_PURPLE: color = [0.5,0,1]
    elif color == GLLIB_WHITE:  color = [1,1,1]
    elif color == GLLIB_PINK:  color = [1,0,1]
    elif color == GLLIB_CYAN:  color = [0,1,1]
    color = [color[0]*intensity,color[1]*intensity,color[2]*intensity]
    color = map(str,color)
    Shader.render_equation("color = vec4("+color[0]+","+color[1]+","+color[2]+",1.0);")
    return Shader
def glLibMaterialPhongShader():
    Shader = glLibShader()
    Shader.render_equation("""
    color += ambient_light0_factor  * ambient_color;
    color += diffuse_light0_factor  * diffuse_color;
    color += specular_light0_factor * specular_color;
    """)
    return Shader
def glLibSingleTexturePhongShader():
    Shader = glLibShader()
    Shader.render_equation("""
    vec4 texsample = tex(tex1,uv);
    color += ambient_light0_factor  * texsample;
    color += diffuse_light0_factor  * texsample;
    color += specular_light0_factor * texsample;
    """)
    return Shader
def glLibDepthBufferShader():
    Shader = glLibShader()
    Shader.render_equation("""
    float z = gl_FragCoord.z;// / gl_FragCoord.w;
    float intensity = z/(gl_DepthRange.far-gl_DepthRange.near);
    color = vec4(intensity,intensity,intensity,1.0);""")
    return Shader
def glLibBlurShader():
    Shader = glLibShader()
    Shader.user_variables("""uniform int stage;
    uniform vec2 uvstep;
    uniform int size;
    uniform sampler2D scenebuffertex1, scenebuffertex2;""")
    Shader.render_equation("""
    float fsize = float(size);
    vec4 blursample = vec4(0.0,0.0,0.0,1.0);
    float intensity;
    float sum = 0.0;
    if (stage==1) {
        for (int i=-size;i<size+1;i++) {
            intensity = 1.0 - (abs(float(i))/(fsize+1.0));
            blursample += tex(scenebuffertex1,uv+vec2(float(i)*uvstep.x,0))*intensity;
            sum += intensity;
        }
    }
    if (stage==2) {
        for (int i=-size;i<size+1;i++) {
            intensity = 1.0 - (abs(float(i))/(fsize+1.0));
            blursample += tex(scenebuffertex2,uv+vec2(0,float(i)*uvstep.y))*intensity;
            sum += intensity;
        }
    }
    blursample = vec4(blursample.rgb/sum,1.0);
    color += blursample;
    """)
    Shader.scenebuffertex1 = Shader.scenebuffertex2 = Shader.scenebuffertex3 = None
    return Shader
def glLibShadowShader():
    Shader = glLibShader()
    Shader.user_variables("""
    varying vec4 ProjCoord;""")
    Shader.render_equation("""
    color += ambient_light0_factor  * ambient_color;
    color += diffuse_light0_factor  * diffuse_color;
    color += specular_light0_factor * specular_color;

    vec4 ShadowValue = shadow2DProj(shadtex1,ProjCoord);
    
    color *= clamp(ShadowValue+0.5,0.0,1.0);
    """)
    Shader.vertex_transform("""
    ProjCoord = gl_TextureMatrix[0]*gl_Vertex;
    """)
    return Shader
GLLIB_SHADER_BLANK = glLibBlankShader()
GLLIB_SHADER_BLUR = glLibBlurShader()
GLLIB_SHADER_DEPTH = glLibDepthBufferShader()
GLLIB_SHADER_RED = glLibColorShader(GLLIB_RED,1.0)
GLLIB_SHADER_ORANGE = glLibColorShader(GLLIB_ORANGE,1.0)
GLLIB_SHADER_YELLOW = glLibColorShader(GLLIB_YELLOW,1.0)
GLLIB_SHADER_GREEN = glLibColorShader(GLLIB_GREEN,1.0)
GLLIB_SHADER_BLUE = glLibColorShader(GLLIB_BLUE,1.0)
GLLIB_SHADER_PURPLE = glLibColorShader(GLLIB_PURPLE,1.0)
GLLIB_SHADER_WHITE = glLibColorShader(GLLIB_WHITE,1.0)
GLLIB_SHADER_BLACK = glLibColorShader(GLLIB_WHITE,0.0)
GLLIB_SHADER_GREY = GLLIB_SHADER_GRAY = glLibColorShader(GLLIB_WHITE,0.2)
GLLIB_SHADER_TEXTURE = glLibSingleTexturePhongShader()
GLLIB_SHADER_MATERIAL = glLibMaterialPhongShader()
GLLIB_SHADER_SHADOW = glLibShadowShader()

blurview = glLibView2D((0,0,1,1))
def glLibBlur(rect,shader,radius,uvstep):
    currenttexture = glGetIntegerv(GL_ACTIVE_TEXTURE) - GL_TEXTURE0

    glLibActiveTexture(9)
    shader.scenebuffertex1 = glLibSceneToTexture(rect,shader.scenebuffertex1)

    blurview.rect = rect
    blurview.set_view()

    glLibUseShader(shader)
    
    glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT)
    glLoadIdentity()
    glBindTexture(GL_TEXTURE_2D,shader.scenebuffertex1)
    glUniform1i(glGetUniformLocation(shader.program,"scenebuffertex1"),9)
    shader.pass_int(1,"stage")
    shader.pass_vec2(uvstep,"uvstep")
    shader.pass_int(radius,"size")
    glLibTexFullScreenQuad()

    glLibActiveTexture(10)
    shader.scenebuffertex2 = glLibSceneToTexture(rect,shader.scenebuffertex2)
    glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT)
    glLoadIdentity()
    glBindTexture(GL_TEXTURE_2D,shader.scenebuffertex2)
    glUniform1i(glGetUniformLocation(shader.program,"scenebuffertex2"),10)
    shader.pass_int(2,"stage")
    glLibTexFullScreenQuad()

    glLibActiveTexture(currenttexture)
depthmap = None
shadowmapsize = 512
LightView = glLibView3D((0,0,shadowmapsize,shadowmapsize),20,9,11.3)
lightpos = [0.01,10,0]
def glLibShadow(): #NEEDS INTENSIVE WORK!
    LightView.set_view()
    LightProjectionMatrix = glGetFloatv(GL_PROJECTION_MATRIX)
    Window.clear()
    gluLookAt(lightpos[0],lightpos[1],lightpos[2],
              0,0,0,
              0,1,0)
    LightViewMatrix = glGetFloatv(GL_MODELVIEW_MATRIX)
    glLightfv(GL_LIGHT0,GL_POSITION,[lightpos[0],lightpos[1],lightpos[2],1.0])

    glLibActiveTexture(8)
    depthmap = glLibSceneToShadowMap(shadowmapsize,depthmap)
    glLibActiveTexture(0)

    View3D.set_view()
    Window.clear()
    camerapos = [cameraradius*cos(radians(camerarot[1]))*cos(radians(camerarot[0])),
                 cameraradius*sin(radians(camerarot[0])),
                 cameraradius*sin(radians(camerarot[1]))*cos(radians(camerarot[0]))]
    gluLookAt(camerapos[0],camerapos[1],camerapos[2],
              0,0,0,
              0,1,0)
    glLightfv(GL_LIGHT0,GL_POSITION,[lightpos[0],lightpos[1],lightpos[2],1.0])
    glLibUseShader(TextureShader)
    
    glMatrixMode(GL_TEXTURE)
    glLoadMatrixf([[0.5,0.0,0.0,0.0],
                   [0.0,0.5,0.0,0.0],
                   [0.0,0.0,0.5,0.0],
                   [0.5,0.5,0.5,1.0]])
    glMultMatrixf(LightProjectionMatrix)
    glMultMatrixf(LightViewMatrix)
    glMatrixMode(GL_MODELVIEW)
    
    glLibUseShader(ShadowShader)
    glLibActiveTexture(8);ShadowShader.pass_shadow_texture(depthmap);glLibActiveTexture(0)
