from glLibLocals import *
from glLibTexturing import *
def CompileShader(source,shaderType):
    shader = glCreateShaderObjectARB(shaderType)
    glShaderSourceARB(shader, source)
    glCompileShaderARB(shader)
    return shader
def glLibUseShader(shader):
    if shader != None:
        try:
            if not shader.compiled:
                raise glLibError("Shader not compiled!")
            else:
                shader = shader.program
        except:
            raise glLibError("Shader not a glLibShader object!")
    else:
        shader = 0
    glUseProgramObjectARB(shader)
class glLibShader:
    def __init__(self):
        self.errors = "Shader not yet compiled!"
        self.compiled = False
        self.renderequation = ""
        self.uservars = ""
        self.verttrans = ""
        self.uvtrans = ""
        self.extfunc = ""
    def get_errors(self):
        return self.errors
    def render_equation(self,renderequation):
        self.renderequation = renderequation
    def user_variables(self,user_vars):
        self.uservars = user_vars
    def vertex_transform(self,verttrans):
        self.verttrans = verttrans
    def uv_transform(self,uvtrans):
        self.uvtrans = uvtrans
    def extension_functions(self,extfuncs):
        self.extfunc = extfuncs
    def compile(self):
        self.source = ShaderSource(self.renderequation,self.uservars,self.verttrans,self.uvtrans,self.extfunc)
        vertex_source = self.source.vert
        fragment_source = self.source.frag
        
        glInitShaderObjectsARB()
        glInitVertexShaderARB()
        glInitFragmentShaderARB()
        
        self.program = glCreateProgramObjectARB()
        
        VertexShader = CompileShader(vertex_source,GL_VERTEX_SHADER_ARB)
        glAttachObjectARB(self.program,VertexShader)
        
        FragmentShader = CompileShader(fragment_source,GL_FRAGMENT_SHADER_ARB)
        glAttachObjectARB(self.program,FragmentShader)
        
        glValidateProgramARB(self.program)
        glLinkProgramARB(self.program)
        
        glDeleteObjectARB(VertexShader)
        glDeleteObjectARB(FragmentShader)
        
        self.errors = glGetInfoLogARB(self.program)
        self.compiled = True
    def pass_texture(self,texture):
        glLibSelectTexture(texture)
        active_texture = glGetIntegerv(GL_ACTIVE_TEXTURE) - GL_TEXTURE0
        glUniform1i(glGetUniformLocation(self.program,"tex"+str(active_texture+1)),active_texture)
    def pass_normalmap(self,texture):
        oldtexture = glGetIntegerv(GL_ACTIVE_TEXTURE) - GL_TEXTURE0
        glLibActiveTexture(6)
        glLibSelectTexture(texture)
        glUniform1i(glGetUniformLocation(self.program,"normalmap"),6)
        glLibActiveTexture(oldtexture)
    def pass_heightmap(self,texture):
        oldtexture = glGetIntegerv(GL_ACTIVE_TEXTURE) - GL_TEXTURE0
        glLibActiveTexture(7)
        glLibSelectTexture(texture)
        glUniform1i(glGetUniformLocation(self.program,"heightmap"),7)
        glLibActiveTexture(oldtexture)
    def pass_shadow_texture(self,texture):
##        glLibSelectTexture(texture)
        glBindTexture(GL_TEXTURE_2D,texture)
        active_texture = glGetIntegerv(GL_ACTIVE_TEXTURE) - GL_TEXTURE8
        glUniform1i(glGetUniformLocation(self.program,"shadtex"+str(active_texture+1)),active_texture+8)
    def pass_float(self,float,name): glUniform1f(glGetUniformLocation(self.program,name),float)
    def pass_int  (self,int,  name): glUniform1i(glGetUniformLocation(self.program,name),int)
    def pass_bool (self,bool, name): glUniform1f(glGetUniformLocation(self.program,name),bool)
    def pass_vec2 (self,vec2, name): glUniform2f(glGetUniformLocation(self.program,name),vec2[0],vec2[1])
    def pass_vec3 (self,vec3, name): glUniform3f(glGetUniformLocation(self.program,name),vec3[0],vec3[1],vec3[2])
    def pass_vec4 (self,vec4, name): glUniform4f(glGetUniformLocation(self.program,name),vec4[0],vec4[1],vec4[2],vec4[3])
class ShaderSource:
    def __init__(self,renderequation,uservars,verttrans,uvtrans,extfunc):
        #VERTEX SHADER SOURCE GENERATION
        self.vert = """
        varying vec3 eyevec;
        varying vec3 LightVec0,LightVec1,LightVec2,LightVec3,LightVec4,LightVec5;
        varying float dist;
        attribute vec4 vert_tangent;
        """+uservars+"""
        void main()
        {	
            vec3 normal = normalize(gl_NormalMatrix * gl_Normal);

            gl_TexCoord[0] = gl_MultiTexCoord0;

            vec4 vertex = gl_Vertex;
            
            vec3 n = normal;
            vec3 t = normalize(gl_NormalMatrix * vert_tangent.xyz);
            vec3 b = cross(n, t)*vert_tangent.w;
            vec3 vVertex = vec3(vec4(gl_ModelViewMatrix*gl_Vertex).xyz);
            vec3 tmpVec;"""
        for lightnum in xrange(6):
            self.vert += "tmpVec = normalize(gl_LightSource[%i].position.xyz-vVertex);" % (lightnum)
            self.vert += "LightVec%i.x = dot(tmpVec, t);" % (lightnum)
            self.vert += "LightVec%i.y = dot(tmpVec, b);" % (lightnum)
            self.vert += "LightVec%i.z = dot(tmpVec, n);" % (lightnum)
        self.vert += """
            eyevec.x = dot(vVertex, t);
            eyevec.y = dot(vVertex, b);
            eyevec.z = dot(vVertex, n);

            """+verttrans+"""

            gl_Position = gl_ModelViewProjectionMatrix * vertex;

            vec3 deltapos = vec3(gl_Position-gl_LightSource[0].position);
            dist = length(deltapos);
        }"""
        #FRAGMENT SHADER SOURCE GENERATION
        self.frag = """
        varying vec3 eyevec;
        varying vec3 LightVec0,LightVec1,LightVec2,LightVec3,LightVec4,LightVec5;
        varying float dist;
        uniform sampler2D tex1,tex2,tex3,tex4,tex5,tex6;
        uniform sampler2D normalmap,heightmap;
        uniform sampler2DShadow shadtex1,shadtex2,shadtex3,shadtex4,shadtex5,shadtex6;
        vec2 uv;
        """+uservars+"""

        vec4 fog(vec4 color)
        {
            float z = gl_FragCoord.z / gl_FragCoord.w;
            z = clamp((z-2.0)/5.0,0.0,1.0);
            float oneminusz = 1.0 - z;
            color = vec4((color[0]*oneminusz)+(1.0*z),(color[1]*oneminusz)+(1.0*z),(color[2]*oneminusz)+(1.0*z),1.0);
            return color;
        }
        vec4 cartoon(vec4 color, int steps)
        {
            float grey = (color.r+color.g+color.g)/3.0;
            float steps2 = float(steps);
            float intensity = float(floor(grey*steps2))/steps2;
            color = vec4(intensity*color.rgb,color.a);
            return color;
        }
        vec4 outline(in vec4 color, in float threshhold, in vec4 setcolor, in float normz)
        {
            if (normz < threshhold) {
                return setcolor;
            }
            return color;
        }
        vec2 parallaxmap(sampler2D heightmap, vec2 texcoords, float scale, bool flip, float add)
        {
            if (flip) {
                float fDepth = (add+texture2D(heightmap,texcoords).r);
                vec2 vHalfOffset = normalize(eyevec).xy * fDepth * scale;
                fDepth = (fDepth+(add+texture2D(heightmap,texcoords+vHalfOffset).r)) * 0.5;
                vHalfOffset = normalize(eyevec).xy * fDepth * scale;
                fDepth = (fDepth+(add+texture2D(heightmap,texcoords+vHalfOffset).r)) * 0.5;
                vHalfOffset = normalize(eyevec).xy * fDepth * scale;
                return texcoords + vHalfOffset;
            }
            else {
                float fDepth = (add+(1.0-texture2D(heightmap,texcoords).r));
                vec2 vHalfOffset = normalize(eyevec).xy * fDepth * scale;
                fDepth = (fDepth+(add+(1.0-texture2D(heightmap,texcoords+vHalfOffset).r))) * 0.5;
                vHalfOffset = normalize(eyevec).xy * fDepth * scale;
                fDepth = (fDepth+(add+(1.0-texture2D(heightmap,texcoords+vHalfOffset).r))) * 0.5;
                vHalfOffset = normalize(eyevec).xy * fDepth * scale;
                return texcoords + vHalfOffset;
            }
        }
        vec4 tex(sampler2D texture, vec2 texcoords)
        {
            return texture2D(texture,texcoords);
        }
        """+extfunc+"""
        void main()
        {
            //color
            vec4 color = vec4(0.0,0.0,0.0,1.0);

            //Tex coords
            uv = gl_TexCoord[0].st;
        """+uvtrans+"""

            //Normal
            vec3 normal = vec3(0.0,0.0,1.0);
            vec3 normalmap_normal = normalize(tex(normalmap,uv).rgb*2.0-1.0);"""
        
        #Color Materials
        if "ambient_color"  in renderequation: self.frag += "vec4 ambient_color =  gl_FrontMaterial.ambient;"
        if "diffuse_color"  in renderequation: self.frag += "vec4 diffuse_color =  gl_FrontMaterial.diffuse;"
        if "specular_color" in renderequation: self.frag += "vec4 specular_color = gl_FrontMaterial.specular;"
        for lightnum in xrange(6):
            #Light Vector
            self.frag += "vec3 lightvec%i = LightVec%i*inversesqrt(dot(LightVec%i,LightVec%i));" % (lightnum,lightnum,lightnum,lightnum)
            #Light Attenuation
            self.frag += "float light%i_att = 1.0/((gl_LightSource[%i].constantAttenuation)+(gl_LightSource[%i].linearAttenuation*dist)+(gl_LightSource[%i].quadraticAttenuation*dist*dist));" % (lightnum,lightnum,lightnum,lightnum)
            #Light Ambient Factors
            self.frag += "vec4 ambient_light%i_factor = gl_LightSource[%i].ambient;" % (lightnum,lightnum)
            #Light Diffuse Factors (Interpolated Vertex Normals)
            self.frag += "vec4 diffuse_light%i_factor = gl_LightSource[%i].diffuse * max(dot(lightvec%i,normal),0.0);" % (lightnum,lightnum,lightnum)
            #Light Diffuse Factors (Normal-map Normals)
            self.frag += "vec4 diffuse_light%i_factor_nm = gl_LightSource[%i].diffuse * max(dot(lightvec%i,normalmap_normal),0.0);" % (lightnum,lightnum,lightnum)
            #Light Specular Factors (Interpolated Vertex Normals)
            self.frag += "vec4 specular_light%i_factor = gl_LightSource[%i].specular * pow(max(dot(lightvec%i,normal),0.0),gl_FrontMaterial.shininess);" % (lightnum,lightnum,lightnum)
            #Light Specular Factors (Normal-map Normals)
            self.frag += "vec4 specular_light%i_factor_nm = gl_LightSource[%i].specular * pow(max(dot(lightvec%i,normalmap_normal),0.0),gl_FrontMaterial.shininess);" % (lightnum,lightnum,lightnum)
            #Total Factors (Interpolated Vertex Normals)
            self.frag += "vec4 total_light%i_factor = light%i_att*(ambient_light%i_factor+diffuse_light%i_factor+specular_light%i_factor);" % (lightnum,lightnum,lightnum,lightnum,lightnum)
            #Total Factors (Normal-map Normals)
            self.frag += "vec4 total_light%i_factor_nm = light%i_att*(ambient_light%i_factor+diffuse_light%i_factor_nm+specular_light%i_factor_nm);" % (lightnum,lightnum,lightnum,lightnum,lightnum)

        self.frag += renderequation+"""

            //clamp color
            color = clamp(color,0.0,1.0);
            
            //set color
            gl_FragColor = color;
        }"""
